--[[--------------------------------------------------------------------------------------------------
MAIN
------------------------------------------------------------------------------------------------------
file: enhancedBunkerSilo
author:	Team LTW
start project: 29.12.2020 
------------------------------------------------------------------------------------------------------
Farming Simulator 19,
V1.0.0.0 - 01.01.2021 - first realease

Farming Simulator 22
V1.0.0.0 - 12.12.2021 - converted to Farming Simulator 22
------------------------------------------------------------------------------------------------------
copyright (c) Team LTW
----------------------------------------------------------------------------------------------------]]

enhancedBunkerSilo = {}

-- load files
local directory = g_currentModDirectory
source(directory.."scripts/BunkerSiloFillAddEvent.lua")

function enhancedBunkerSilo:deleteMap()
end 

function enhancedBunkerSilo:mouseEvent(posX, posY, isDown, isUp, button)
end 

function enhancedBunkerSilo:keyEvent(unicode, sym, modifier, isDown)
end 

function enhancedBunkerSilo:update(dt)	
end 

function enhancedBunkerSilo:draw()	
end

-- New functions for BunkerSilo
------------------------------------------

function BunkerSilo:getCanFillAddSilo()
	return self.state == BunkerSilo.STATE_DRAIN and self.fillLevel > self.emptyThreshold
end

-- Override functions
------------------------------------------

function enhancedBunkerSilo.BunkerSiloSetState(self, oldFunc, state, showNotification)
	if state ~= self.state then
		if state == BunkerSilo.STATE_FILL then
			-- new code +++
			if self.fillLevel > self.emptyThreshold then
				self.compactedFillLevel = self.fillLevel
				if self.isServer then
					local area = self.bunkerSiloArea
					local offsetFront = {self:getBunkerAreaOffset(true, 0, self.fermentingFillType),self:getBunkerAreaOffset(true, 0, self.outputFillType)}
					local offsetBack = {self:getBunkerAreaOffset(false, 0, self.fermentingFillType),self:getBunkerAreaOffset(false, 0, self.outputFillType)}
					local fillType = {self.fermentingFillType,self.outputFillType}
					for i=1, 2, 1 do
						local x0 = area.sx + offsetFront[i] * area.dhx_norm
						local z0 = area.sz + offsetFront[i] * area.dhz_norm
						local x1 = x0 + area.dwx
						local z1 = z0 + area.dwz
						local x2 = area.sx + area.dhx - offsetBack[i] * area.dhx_norm
						local z2 = area.sz + area.dhz - offsetBack[i] * area.dhz_norm
						local changed = DensityMapHeightUtil.changeFillTypeAtArea(x0, z0, x1, z1, x2, z2, fillType[i], self.inputFillType)
					end
				end
				if showNotification then
					self:showBunkerMessage(g_i18n:getText("ingameNotification_bunkerSiloIsOpenComplettly"))
				end
			else
				self.compactedFillLevel = 0
				self.compactedPercent = 0
				if showNotification then
					self:showBunkerMessage(g_i18n:getText("ingameNotification_bunkerSiloIsEmpty"))
				end
			end			
			self.fermentingTime = 0
			self.fermentingPercent = 0
			self.isOpenedAtFront = false
			self.isOpenedAtBack = false
			self.bunkerSiloArea.offsetFront = 0
			self.bunkerSiloArea.offsetBack = 0
			-- new code ---
			if self.isServer then
				g_densityMapHeightManager:removeFixedFillTypesArea(self.bunkerSiloArea)
				g_densityMapHeightManager:setConvertingFillTypeAreas(self.bunkerSiloArea, self.acceptedFillTypes, self.inputFillType)
			end

		elseif state == BunkerSilo.STATE_CLOSED then
			if self.isServer then
				local area = self.bunkerSiloArea
				local offsetFront = self:getBunkerAreaOffset(true, 0, self.inputFillType)
				local offsetBack = self:getBunkerAreaOffset(false, 0, self.inputFillType)
				local x0 = area.sx + offsetFront * area.dhx_norm
				local z0 = area.sz + offsetFront * area.dhz_norm
				local x1 = x0 + area.dwx
				local z1 = z0 + area.dwz
				local x2 = area.sx + area.dhx - offsetBack * area.dhx_norm
				local z2 = area.sz + area.dhz - offsetBack * area.dhz_norm
				local changed = DensityMapHeightUtil.changeFillTypeAtArea(x0, z0, x1, z1, x2, z2, self.inputFillType, self.fermentingFillType)

				g_densityMapHeightManager:removeFixedFillTypesArea(self.bunkerSiloArea)
				g_densityMapHeightManager:removeConvertingFillTypeAreas(self.bunkerSiloArea)
			end

			if showNotification then
				self:showBunkerMessage(g_i18n:getText("ingameNotification_bunkerSiloCovered"))
			end
		elseif state == BunkerSilo.STATE_FERMENTED then
			if showNotification then
				self:showBunkerMessage(g_i18n:getText("ingameNotification_bunkerSiloDoneFermenting"))
			end
		elseif state == BunkerSilo.STATE_DRAIN then
			self.bunkerSiloArea.offsetFront = 0
			self.bunkerSiloArea.offsetBack = 0

			if showNotification then
				self:showBunkerMessage(g_i18n:getText("ingameNotification_bunkerSiloOpened"))
			end

			if self.isServer then
				g_densityMapHeightManager:removeConvertingFillTypeAreas(self.bunkerSiloArea)

				local fillTypes = {
					[self.outputFillType] = true
				}

				g_densityMapHeightManager:setFixedFillTypesArea(self.bunkerSiloArea, fillTypes)
			end

		end

		self.state = state
		if self.isServer then
			self:raiseDirtyFlags(self.bunkerSiloDirtyFlag)
		end
	end
end

function enhancedBunkerSilo.BunkerSiloGetIsActivatable(self, oldFunc)
	if self.bunkerSilo:getCanInteract() and (self.bunkerSilo:getCanCloseSilo() or self.bunkerSilo:getCanOpenSilo() or self.bunkerSilo:getCanFillAddSilo()) then
		self:updateActivateText()

		return true
	end

	return false
end

function enhancedBunkerSilo.BunkerSiloRun(self, oldFunc)
	if self.bunkerSilo:getCanCloseSilo() then
		if g_server ~= nil then
			self.bunkerSilo:setState(BunkerSilo.STATE_CLOSED, true)
		else
			g_client:getServerConnection():sendEvent(BunkerSiloCloseEvent.new(self.bunkerSilo))
		end
	elseif self.bunkerSilo:getCanOpenSilo() then
		local ix, iy, iz = self.bunkerSilo:getInteractionPosition()

		if ix ~= nil then
			if g_server ~= nil then
				self.bunkerSilo:openSilo(ix, iy, iz)
			else
				g_client:getServerConnection():sendEvent(BunkerSiloOpenEvent.new(self.bunkerSilo, ix, iy, iz))
			end
		end
	-- new code +++
	elseif self.bunkerSilo:getCanFillAddSilo() then
		if g_server ~= nil then
			self.bunkerSilo:setState(BunkerSilo.STATE_FILL, true)
		else
			g_client:getServerConnection():sendEvent(BunkerSiloFillAddEvent.new(self.bunkerSilo))
		end
	-- new code ---
	end

	self:updateActivateText()
end

function enhancedBunkerSilo.BunkerSiloUpdateActivateText(self, oldFunc)
	self.activateText = "unknown"
	local ix, iy, iz = self.bunkerSilo:getInteractionPosition()
	local closerToFront = self.bunkerSilo:getIsCloserToFront(ix, iy, iz)

	if self.bunkerSilo.state == BunkerSilo.STATE_FILL then
		self.activateText = g_i18n:getText("action_blanketSilo")
	elseif self.bunkerSilo.state == BunkerSilo.STATE_FERMENTED then
		self.activateText = g_i18n:getText("action_openSilo")
	elseif self.bunkerSilo.state == BunkerSilo.STATE_DRAIN and ((not self.bunkerSilo.isOpenedAtFront and closerToFront) or (not self.bunkerSilo.isOpenedAtBack and not closerToFront)) then
		self.activateText = g_i18n:getText("action_openSilo")
	elseif self.bunkerSilo.state == BunkerSilo.STATE_DRAIN and ((self.bunkerSilo.isOpenedAtFront and closerToFront) or (self.bunkerSilo.isOpenedAtBack and not closerToFront)) then
		self.activateText = g_i18n:getText("action_open_silo_complettly")
	end
end


--- Override functions on first load
local function init()
	BunkerSilo.setState = Utils.overwrittenFunction(BunkerSilo.setState, enhancedBunkerSilo.BunkerSiloSetState)
	BunkerSiloActivatable.run = Utils.overwrittenFunction(BunkerSiloActivatable.run, enhancedBunkerSilo.BunkerSiloRun)	
	BunkerSiloActivatable.getIsActivatable = Utils.overwrittenFunction(BunkerSiloActivatable.getIsActivatable, enhancedBunkerSilo.BunkerSiloGetIsActivatable)	
	BunkerSiloActivatable.updateActivateText = Utils.overwrittenFunction(BunkerSiloActivatable.updateActivateText, enhancedBunkerSilo.BunkerSiloUpdateActivateText)	
	print("Init enhanced Bunker Silo -- by Team LTW")
end

init()
